package com.qizhen.healsphere.common.util;


import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 大洲转换工具类
 * @author: dsYun
 * @time: 2025/04/27 14:21
 */
public class ContinentConverter {

    // 大洲中英文映射表（英文小写 + 下划线格式）
    private static final Map<String, String> CONTINENT_MAP;
    static {
        Map<String, String> map = new HashMap<>();
        map.put("asia", "亚洲");
        map.put("europe", "欧洲");
        map.put("africa", "非洲");
        map.put("north_america", "北美洲");
        map.put("south_america", "南美洲");
        map.put("oceania", "大洋洲");
        map.put("antarctica", "南极洲"); // 可选
        CONTINENT_MAP = Collections.unmodifiableMap(map); // 不可变 Map
    }

    /**
    * 将英文大洲名称转换为中文（支持多个大洲逗号分隔）
    * @param englishNames 如 "North America, Oceania"
    * @return 如 "北美洲, 大洋洲"
    */
    public static String toChinese(String englishNames) {
        if (englishNames == null || englishNames.isEmpty()) {
            return "未知地区";
        }
        String[] parts = englishNames.split(",\\s*");
        StringBuilder result = new StringBuilder();
        for (String part : parts) {
            // 统一处理格式：小写 + 替换空格为下划线
            String key = part.trim().toLowerCase().replace(" ", "_");
            String chinese = CONTINENT_MAP.getOrDefault(key, part + "(未知)");
            result.append(chinese).append(", ");
        }
        if (result.length() > 0) {
            result.setLength(result.length() - 2); // 移除末尾的 ", "
        }
        return result.toString();
    }
}
