package com.qizhen.healsphere.common.util;


import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 省份转换工具类
 * @author: dsYun
 * @time: 2025/04/27 14:21
 */
public class ProvinceConverter {

    private static final Map<String, String> PROVINCE_MAP;
    static {
        Map<String, String> map = new HashMap<>();
        map.put("anhui", "安徽");
        map.put("beijing", "北京");
        map.put("chongqing", "重庆");
        map.put("fujian", "福建");
        map.put("gansu", "甘肃");
        map.put("guangdong", "广东");
        map.put("guangxi", "广西");
        map.put("guizhou", "贵州");
        map.put("hainan", "海南");
        map.put("hebei", "河北");
        map.put("heilongjiang", "黑龙江");
        map.put("henan", "河南");
        map.put("hubei", "湖北");
        map.put("hunan", "湖南");
        map.put("neimenggu", "内蒙古");
        map.put("jiangsu", "江苏");
        map.put("jiangxi", "江西");
        map.put("jilin", "吉林");
        map.put("liaoning", "辽宁");
        map.put("ningxia", "宁夏");
        map.put("qinghai", "青海");
        map.put("shaanxi", "陕西");
        map.put("shandong", "山东");
        map.put("shanghai", "上海");
        map.put("shanxi", "山西");
        map.put("sichuan", "四川");
        map.put("taiwan", "台湾");
        map.put("tianjin", "天津");
        map.put("tibet", "西藏");
        map.put("xinjiang", "新疆");
        map.put("yunnan", "云南");
        map.put("zhejiang", "浙江");
        // 特别行政区
        map.put("hong_kong", "香港");
        map.put("macao", "澳门");
        PROVINCE_MAP = Collections.unmodifiableMap(map); // 不可变 Map
    }

    /**
    * 将英文省份转换为中文（支持多个地区逗号分隔）
    * @param englishNames 如 "Yunnan, Guangdong"
    * @return 如 "云南, 广东"
    */
    public static String toChinese(String englishNames) {
        if (englishNames == null || englishNames.isEmpty()) {
            return "未知地区";
        }
        String[] parts = englishNames.split(",\\s*");
        StringBuilder result = new StringBuilder();
        for (String part : parts) {
            String key = part.trim().toLowerCase();
            String chinese = PROVINCE_MAP.getOrDefault(key, part + "(未知)");
            result.append(chinese).append(", ");
        }
        if (result.length() > 0) {
            result.setLength(result.length() - 2); // 移除末尾的 ", "
        }
        return result.toString();
    }
}
