package com.qizhen.healsphere.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qizhen.healsphere.common.util.BeanUtil;
import com.qizhen.healsphere.common.util.ContinentConverter;
import com.qizhen.healsphere.common.util.CountryConverter;
import com.qizhen.healsphere.common.util.ProvinceConverter;
import com.qizhen.healsphere.repository.mapper.EpifluMapper;
import com.qizhen.healsphere.repository.mapper.EpifluQueryMapper;
import com.qizhen.healsphere.repository.mapper.entity.Isolate;
import com.qizhen.healsphere.service.impl.IsolateServiceImpl;
import com.qizhen.healsphere.web.dto.*;
import com.qizhen.healsphere.web.param.*;
import com.qizhen.healsphere.web.vo.ChinaInputVO;
import com.qizhen.healsphere.web.vo.InputVO;
import org.apache.poi.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @Description: 用户日志业务层
 * @author: gaodm
 * @time: 2018/8/6 9:11
 */
@Component
public class IsolateFacade extends IsolateServiceImpl {

    @Autowired
    private EpifluMapper epifluMapper;

    @Autowired
    private EpifluQueryMapper epifluQueryMapper;

    /**
     * 获取参数
     *
     * @param inputVO
     * @return List<GraphInfo>
     */
    public List<IsolateDTO> getIsolate(InputVO inputVO) {
        List<IsolateDTO> resultList = new ArrayList<>();
        List<EpifluDTO> subtypeContinent = epifluMapper.getSubtypeContinent(inputVO);

//        boolean flag = StringUtil.isBlank(inputVO.getSubtype())
//                || subtypeContinent.stream()
//                .map(EpifluDTO::getSubtype)
//                .noneMatch(st -> Objects.equals(st, inputVO.getSubtype()));
//
//        if (flag) {
//            for (EpifluDTO epifluDTO : subtypeContinent) {
//                resultList.add(getIsolateDTOList(inputVO, epifluDTO));
//            }
//        } else {
//            // 创建并填充DTO
//            IsolateDTO isolateDTO = new IsolateDTO();
//            isolateDTO.setEpifluentName("甲流");
//            resultList.add(isolateDTO);
//        }
        for (EpifluDTO epifluDTO : subtypeContinent) {
            resultList.add(getIsolateDTOList(inputVO, epifluDTO));
        }

        return resultList;
    }

    public IsolateDTO getIsolateDTOList(InputVO inputVO, EpifluDTO epifluDTO) {

        IsolateDTO isolateDTO = new IsolateDTO();

        // 定义日期格式
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        String subtype = epifluDTO.getSubtype();
        QueryWrapper<Epiflu> wrapper = new QueryWrapper<>();
        wrapper.eq("subtype", subtype)
                .orderByAsc("subtype", "collection_date");
        List<Epiflu> epiflus = epifluMapper.selectList(wrapper);

        if (epiflus.isEmpty()) {
            return isolateDTO;
        }
        BeanUtil.copyProperties(epifluDTO, isolateDTO);
        isolateDTO.setEpifluentName("甲流");
        // 处理首个隔离信息
        Epiflu firstEpiflu = epiflus.get(0);
        String continent = getSegment(firstEpiflu.getLocation(), 0);//大洲
        String continentChinese = ContinentConverter.toChinese(continent);
        String country = getSegment(firstEpiflu.getLocation(), 1);//国家
        String countryChinese= CountryConverter.toChinese(country);
//        String firstLocation = firstEpiflu.getLocation() != null ? firstEpiflu.getLocation() : "未知地区";
        Date firstDate = firstEpiflu.getCollectionDate();
        // 格式化日期（处理空值）
        String formattedDate = (firstDate != null) ? dateFormat.format(firstDate) : "未知日期";

        // 收集传播地点（排除首个）
        Set<String> propagationLocations = new LinkedHashSet<>();
        for (int i = 1; i < epiflus.size(); i++) {
            String location = epiflus.get(i).getLocation();
            if (location != null) {
//                String lastSegment = getLastSegment(location);
                String continent2 = getSegment(location, 0);//大洲
                String continentChinese2 = ContinentConverter.toChinese(continent2);
                String country2 = getSegment(location, 1);//国家
                String countryChinese2= CountryConverter.toChinese(country2);
                propagationLocations.add(continentChinese2+countryChinese2);
            }
        }

        String propagation = propagationLocations.isEmpty() ? "无" : String.join(",", propagationLocations);
        // 构建分析文本
        String analyse = String.format("%s首次爆发在%s的%s%s，之后在%s传播。",
                subtype, formattedDate, continentChinese,countryChinese,propagation);

        // 创建并填充DTO
        isolateDTO.setEpifluentName("甲流");
        isolateDTO.setCumulativeMap(getCumulativeList(inputVO, epifluDTO));
        isolateDTO.setAnalyse(analyse);
        isolateDTO.setGeoInfoList(getGeoInfoList(inputVO, epifluDTO));

        return isolateDTO;

    }

    public Map<String, CumulativeDTO> getCumulativeList(InputVO inputVO, EpifluDTO epifluDTO) {
        String cladeStr = epifluDTO.getClade();
        List<String> cladeList = Optional.ofNullable(cladeStr)
                .filter(s -> !s.trim().isEmpty())
                .map(s -> Arrays.stream(s.split(","))
                        .map(String::trim)          // 去除前后空格
                        .filter(part -> !part.isEmpty()) // 过滤空字符串
                        .collect(Collectors.toList())    // 改为收集为 List
                )
                .orElse(Collections.emptyList());        // 默认返回空 List
        Map<String, CumulativeDTO> cumulativeMap = new HashMap<>();

        ChinaCollectionDateDTO chinaCollectionDateDTO = new ChinaCollectionDateDTO();
        if (StringUtil.isNotBlank(inputVO.getStartDate()) && StringUtil.isNotBlank(inputVO.getEndDate())) {
            chinaCollectionDateDTO.setMINCollectionDate(inputVO.getStartDate());
            chinaCollectionDateDTO.setMAXCollectionDate(inputVO.getEndDate());
            chinaCollectionDateDTO.setCount(epifluDTO.getCount());
        } else {
            String subtype = epifluDTO.getSubtype();
            chinaCollectionDateDTO = epifluMapper.findCollectionDate(subtype);
        }

        // 定义起始和结束日期
        LocalDate startDate = LocalDate.parse(chinaCollectionDateDTO.getMINCollectionDate());
        LocalDate endDate = LocalDate.parse(chinaCollectionDateDTO.getMAXCollectionDate());
        // 生成日期列表
        List<LocalDate> dateList = generateDateIntervals(startDate, endDate, 30);

        for (LocalDate localDate : dateList) {
            CumulativeDTO cumulativeDTO = new CumulativeDTO();
            inputVO.setEndDate(localDate.toString());
            inputVO.setSubtype(epifluDTO.getSubtype());
            inputVO.setClade(cladeList);
            Integer count = epifluMapper.selectCountByInput(inputVO);
//            double percentage = (chinaCollectionDateDTO.getCount() == 0) ? 0.0 : (count * 100.0) / chinaCollectionDateDTO.getCount();
            // 计算百分比并保留两位小数
            double percentage = (chinaCollectionDateDTO.getCount() == 0) ? 0.0 :
                    new BigDecimal((count * 100.0) / chinaCollectionDateDTO.getCount())
                            .setScale(2, RoundingMode.HALF_UP)  // 保留两位小数（四舍五入）
                            .doubleValue();
            cumulativeDTO.setCount(count);
            cumulativeDTO.setPercentage(percentage);
            cumulativeMap.put(localDate.toString(), cumulativeDTO);
        }
        return cumulativeMap;
    }

    public List<GeoInfo> getGeoInfoList(InputVO inputVO, EpifluDTO epifluDTO) {
        List<String> continentList = StringUtil.isBlank(inputVO.getContinent())
                ? Arrays.asList("Asia", "Europe", "Africa", "North America", "South America", "Oceania")
                : Collections.singletonList(inputVO.getContinent().trim());

        List<GeoInfo> geoInfoList = new ArrayList<>();

        for (String continent : continentList) {
            GeoInfo geoInfo = new GeoInfo();

            // 1. 获取clade统计数据
            List<CladeCountResult> cladeCountResult = epifluMapper.selectCladeCountByContinent(continent, epifluDTO.getSubtype());

            // 2. 转换数据格式
            List<GeoDataItem> dataItems = cladeCountResult.stream()
                    .map(result -> new GeoDataItem(result.getClade(), result.getCount()))
                    .collect(Collectors.toList());

            // 3. 设置中心坐标（注意这里传入的是Double数组）
            ContinentCoordinates coordinates = ContinentCoordinates.fromName(continent.toUpperCase().replace(" ", "_"));
            geoInfo.setContinentName(continent);
            geoInfo.setCenter(new Double[]{
                    Double.parseDouble(coordinates.getLongitude()),
                    Double.parseDouble(coordinates.getLatitude())
            });

            // 4. 设置数据
            geoInfo.setData(dataItems);

            geoInfoList.add(geoInfo);
        }
        return geoInfoList;

    }


    public List<Epiflu> getSubtypeContinent(String continent) {
        QueryWrapper<Epiflu> wrapper = new QueryWrapper<>();
        wrapper.select("subtype", "clade", "COUNT(*) AS count")
                .eq("continent_location", continent)
                .groupBy("subtype", "clade")
                .orderByDesc("count");

        return epifluMapper.selectList(wrapper);
    }


    /**
     * 获取中国的病毒分布情况
     *
     * @param chinaInputVO
     * @return List<GraphInfo>
     */
    public List<ChinaIsolateDTO> getChinaIsolate(ChinaInputVO chinaInputVO) {
        List<ChinaIsolateDTO> resultList = new ArrayList<>();
        List<ChinaEpifluDTO> chinaEpiflu = epifluMapper.findChinaEpiflu(chinaInputVO);

        if (chinaEpiflu == null) {
            return resultList;
        }

        for (ChinaEpifluDTO chinaEpifluDTO : chinaEpiflu) {
            ChinaIsolateDTO chinaIsolateDTO = new ChinaIsolateDTO();
            BeanUtil.copyProperties(chinaEpifluDTO, chinaIsolateDTO);
            chinaIsolateDTO.setEpifluentName("甲流");


            // 定义日期格式
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
            String subtype = chinaEpifluDTO.getSubtype();
            QueryWrapper<Epiflu> wrapper = new QueryWrapper<>();
            wrapper.eq("subtype", subtype).eq("is_china", 1)
                    .orderByAsc("subtype", "collection_date");
            List<Epiflu> epiflus = epifluMapper.selectList(wrapper);

            if (epiflus.isEmpty()) {
                continue; // 跳过无数据的subtype
            }

            // 处理首个隔离信息
            Epiflu firstEpiflu = epiflus.get(0);
//            String firstLocation = firstEpiflu.getChinaCity() != null ? firstEpiflu.getChinaCity() : "未知地区";
            String firstLocation = ProvinceConverter.toChinese(firstEpiflu.getChinaCity());
            Date firstDate = firstEpiflu.getCollectionDate();
            // 格式化日期（处理空值）
            String formattedDate = (firstDate != null) ? dateFormat.format(firstDate) : "未知日期";

            // 收集传播地点（排除首个）
            Set<String> propagationLocations = new LinkedHashSet<>();
            for (int i = 1; i < epiflus.size(); i++) {
//                String chinaCity = epiflus.get(i).getChinaCity();
                String chinaCity = ProvinceConverter.toChinese(epiflus.get(i).getChinaCity());
                if (chinaCity != null) {
//                    String lastSegment = getLastSegment(location);
                    propagationLocations.add(chinaCity);
                }
            }
            String propagation = propagationLocations.isEmpty() ? "无" : String.join(",", propagationLocations);
            // 构建分析文本
            String analyse = String.format("%s首次爆发在%s的%s，之后在%s传播。",
                    subtype, formattedDate, firstLocation, propagation);

            chinaIsolateDTO.setAnalyse(analyse); //插入说明

            chinaIsolateDTO.setCumulativeMap(getChinaCumulativeList(chinaInputVO, chinaEpifluDTO));//获取各个时间段的数量

            chinaIsolateDTO.setGeoInfoList(chinaGeoInfoList(chinaInputVO, chinaEpifluDTO));//获取中国省份坐标及相关进化分支参数

            resultList.add(chinaIsolateDTO);
        }

        return resultList;
    }


    public List<GeoInfo> chinaGeoInfoList(ChinaInputVO chinaInputVO, ChinaEpifluDTO chinaEpifluDTO) {
        // 处理 clade 参数
        String cladeStr = chinaEpifluDTO.getClade();
        Set<String> cladeSet = Optional.ofNullable(cladeStr)
                .filter(s -> !s.trim().isEmpty())
                .map(s -> Arrays.stream(s.split(","))
                        .map(String::trim)
                        .filter(part -> !part.isEmpty())
                        .collect(Collectors.toSet()))
                .orElse(Collections.emptySet());

        // 构建查询条件
        QueryWrapper<Epiflu> epifluWrapper = new QueryWrapper<Epiflu>()
                .eq("is_china", 1)
                .eq("subtype", chinaEpifluDTO.getSubtype());
        if (!cladeSet.isEmpty()) {
            epifluWrapper.in("clade", cladeSet);
        }

        // 查询并提取结果
        List<String> provinceList = epifluMapper.selectList(epifluWrapper).stream()
                .map(Epiflu::getChinaCity)
                .filter(Objects::nonNull)
                .distinct()
                .collect(Collectors.toList());

        List<String> continentList = StringUtil.isBlank(chinaInputVO.getProvince())
                ? provinceList
                : Collections.singletonList(chinaInputVO.getProvince().trim());

        List<GeoInfo> geoInfoList = new ArrayList<>();

        for (String continent : continentList) {
            GeoInfo geoInfo = new GeoInfo();

            // 1. 获取clade统计数据
            List<CladeCountResult> cladeCountResult = epifluMapper.selectCladeCountByProvince(continent, chinaEpifluDTO.getSubtype());

            // 2. 转换数据格式
            List<GeoDataItem> dataItems = cladeCountResult.stream()
                    .map(result -> new GeoDataItem(result.getClade(), result.getCount()))
                    .collect(Collectors.toList());

            // 3. 设置中心坐标（注意这里传入的是Double数组）
            ChinaProvinceCoordinates ChinaCoordinates = ChinaProvinceCoordinates.fromName(continent.toUpperCase().replace(" ", "_"));
            geoInfo.setContinentName(continent);
            geoInfo.setCenter(new Double[]{
                    Double.parseDouble(ChinaCoordinates.getLongitude()),
                    Double.parseDouble(ChinaCoordinates.getLatitude())
            });

            // 4. 设置数据
            geoInfo.setData(dataItems);

            geoInfoList.add(geoInfo);
        }
        return geoInfoList;
    }


    public Map<String, CumulativeDTO> getChinaCumulativeList(ChinaInputVO chinaInputVO, ChinaEpifluDTO chinaEpifluDTO) {
        String cladeStr = chinaEpifluDTO.getClade();
        List<String> cladeList = Optional.ofNullable(cladeStr)
                .filter(s -> !s.trim().isEmpty())
                .map(s -> Arrays.stream(s.split(","))
                        .map(String::trim)          // 去除前后空格
                        .filter(part -> !part.isEmpty()) // 过滤空字符串
                        .collect(Collectors.toList())    // 改为收集为 List
                )
                .orElse(Collections.emptyList());        // 默认返回空 List
        Map<String, CumulativeDTO> cumulativeMap = new HashMap<>();

        ChinaCollectionDateDTO chinaCollectionDateDTO = new ChinaCollectionDateDTO();
        if (StringUtil.isNotBlank(chinaInputVO.getStartDate()) && StringUtil.isNotBlank(chinaInputVO.getEndDate())) {
            chinaCollectionDateDTO.setMINCollectionDate(chinaInputVO.getStartDate());
            chinaCollectionDateDTO.setMAXCollectionDate(chinaInputVO.getEndDate());
            chinaCollectionDateDTO.setCount(chinaEpifluDTO.getCount());
        } else {
            String subtype = chinaEpifluDTO.getSubtype();
            chinaCollectionDateDTO = epifluMapper.findChinaCollectionDate(subtype);
        }

        // 定义起始和结束日期
        LocalDate startDate = LocalDate.parse(chinaCollectionDateDTO.getMINCollectionDate());
        LocalDate endDate = LocalDate.parse(chinaCollectionDateDTO.getMAXCollectionDate());
        // 生成日期列表
        List<LocalDate> dateList = generateDateIntervals(startDate, endDate, 30);

        for (LocalDate localDate : dateList) {
            CumulativeDTO cumulativeDTO = new CumulativeDTO();
            chinaInputVO.setEndDate(localDate.toString());
            chinaInputVO.setSubtype(chinaEpifluDTO.getSubtype());
            chinaInputVO.setClade(cladeList);
            Integer count = epifluMapper.selectCountByChinaInput(chinaInputVO);
//            double percentage = (chinaCollectionDateDTO.getCount() == 0) ? 0.0 : (count * 100.0) / chinaCollectionDateDTO.getCount();
            double percentage = (chinaCollectionDateDTO.getCount() == 0) ? 0.0 :
                    new BigDecimal((count * 100.0) / chinaCollectionDateDTO.getCount())
                            .setScale(2, RoundingMode.HALF_UP)  // 保留两位小数（四舍五入）
                            .doubleValue();
            cumulativeDTO.setCount(count);
            cumulativeDTO.setPercentage(percentage);
            cumulativeMap.put(localDate.toString(), cumulativeDTO);
        }
        return cumulativeMap;
    }


    public static String getLastSegment(String input) {
        Pattern pattern = Pattern.compile(".*/([^/]+)/?");
        Matcher matcher = pattern.matcher(input);
        return matcher.find() ? matcher.group(1) : "";
    }

    public static String getSegment(String input, int index) {
        if (input == null || input.isEmpty()) {
            return "";
        }
        String[] segments = input.split("/");
        return (index >= 0 && index < segments.length) ? segments[index] : "";
    }


    /**
     * 生成固定间隔的日期列表（包含起始和结束日期）
     *
     * @param start 起始日期
     * @param end   结束日期
     * @param days  间隔天数
     * @return 日期列表
     */
    public static List<LocalDate> generateDateIntervals(LocalDate start, LocalDate end, int days) {
        List<LocalDate> dates = new ArrayList<>();

        // 添加起始日期
        dates.add(start);

        // 计算并添加后续日期
        LocalDate current = start;
        while (true) {
            current = current.plusDays(days);
            if (current.isAfter(end)) {
                break;
            }
            dates.add(current);
        }

        // 确保结束日期被包含（如果未被添加）
        if (!dates.get(dates.size() - 1).equals(end)) {
            dates.add(end);
        }

        return dates;
    }
}
