package com.qizhen.healsphere.repository.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.qizhen.healsphere.web.dto.*;
import com.qizhen.healsphere.web.param.CladeCountResult;
import com.qizhen.healsphere.web.param.Epiflu;
import com.qizhen.healsphere.web.vo.ChinaInputVO;
import com.qizhen.healsphere.web.vo.InputVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.time.LocalDate;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Mapper
public interface EpifluMapper extends BaseMapper<Epiflu> {
    // 获取subtype的列表
    @Select("SELECT DISTINCT subtype FROM epiflu")
    List<String> findDistinctSubtype();

    @Select("SELECT DISTINCT clade FROM epiflu WHERE subtype = #{subtype}")
    List<String> findDistinctCladeBySubtype(@Param("subtype") String subtype);

    @Select("SELECT COUNT(1) FROM epiflu WHERE subtype = #{subtype}")
    Integer selectCountBySubtype(@Param("subtype") String subtype);

    @Select("SELECT MAX(collection_date) FROM epiflu WHERE subtype = #{subtype}")
    Date findMaxCollectionDateBySubtype(@Param("subtype") String subtype);

    @Select("SELECT COUNT(1) FROM epiflu WHERE subtype = #{subtype} AND collection_date BETWEEN #{startDate} AND #{endDate}")
    Integer selectCountBySubtypeAndDateRange(@Param("subtype") String subtype, @Param("startDate") LocalDate startDate, @Param("endDate") LocalDate endDate);

    @Select("SELECT COUNT(1) FROM epiflu WHERE subtype = #{subtype} AND collection_date BETWEEN #{startDate} AND #{endDate} AND location LIKE CONCAT('%', #{continent}, '%')")
    Integer selectCountBySubtypeAndDateRangeWithContinent(@Param("subtype") String subtype, @Param("startDate") LocalDate SeasonStartDate, @Param("endDate") LocalDate SeasonEndDate, @Param("continent") String continent);

//    @Select("<script>" +
//            "SELECT DISTINCT clade " +
//            "FROM epiflu " +
//            "WHERE subtype = #{subtype} " +
//            "AND is_china = #{isChina} " +
//            "AND collection_date BETWEEN #{startDate} AND #{endDate} " +
//            "<if test='constellation != null and constellation.size() > 0'>" +
//            "</script>")
//    List<String> findDistinctCladeBySearchWithConditions(@Param("subtype") String subtype, @Param("isChina") Integer isChina, @Param("startDate") LocalDate startDate, @Param("endDate") LocalDate endDate);

    /**
     * 查询指定洲的clade分布
     * @param continent 洲名称
     * @return clade统计列表
     */
    @Select("SELECT clade, COUNT(*) AS count " +
            "FROM public.epiflu " +
            "WHERE continent_location = #{continent} " +
            "AND subtype = #{subtype} " +
            "GROUP BY clade " +
            "ORDER BY count DESC")
    List<CladeCountResult> selectCladeCountByContinent(@Param("continent") String continent , @Param("subtype") String subType);

    /**
     * 查询指定日期、子类型和星座的记录数
     * @param subtype 子类型
     * @param constellation 星座
     * @param date 日期
     * @return 记录数
     */
    @Select("SELECT COUNT(*) FROM epiflu WHERE subtype = #{subtype} AND clade = #{constellation} AND collection_date = #{date}")
    Integer selectDayCount(@Param("subtype") String subtype, @Param("constellation") String constellation, @Param("date") LocalDate date);

    @Select("SELECT DISTINCT subtype FROM epiflu WHERE subtype = #{is_china}")
    List<String> findDistinctSubtypeChina();

    @Select("SELECT COUNT(*) FROM epiflu WHERE subtype = #{subtype} AND is_china = #{is_china} AND collection_date = #{startDate}")
    Integer selectCount(@Param("subtype") String subtype, @Param("is_china") Integer is_china, @Param("startDate") LocalDate startDate);


@Select("<script>" +
            "SELECT " +
            "subtype, " +
            "STRING_AGG(DISTINCT clade::text, ',') AS clade, " +
            "COUNT(1) AS count " +
            "FROM epiflu " +
            "<where> " +
            "1=1 " +
            "<if test=\"inputVO.continent != null and inputVO.continent != ''\"> " +
            "AND continent_location = #{inputVO.continent} " +
            "</if> " +
            "<if test=\"inputVO.clade != null and inputVO.hasValidClade()\"> " +
            "AND clade in " +
            "<foreach collection=\"inputVO.clade\" item=\"item\" open=\"(\" close=\")\" separator=\",\"> " +
            "#{item}" +
            "</foreach>" +
            "</if> " +
            "<if test=\"inputVO.subtype != null and inputVO.subtype != ''\"> " +
            "AND subtype = #{inputVO.subtype} " +
            "</if> " +
            "<if test=\"inputVO.startDate != null and inputVO.startDate != ''\"> " +
            "AND collection_date >= #{inputVO.startDate}::date " +
            "</if> " +
            "<if test=\"inputVO.endDate != null and inputVO.startDate != ''\"> " +
            "AND collection_date &lt;= #{inputVO.endDate}::date " +
            "</if> " +
            "</where> " +
            "GROUP BY subtype;" +
            "</script>")
    List<EpifluDTO> getSubtypeContinent(@Param("inputVO") InputVO inputVO);

    @Select("<script>" +
            "SELECT " +
            "subtype, " +
            "STRING_AGG(DISTINCT clade::text, ',') AS clade, " +
            "COUNT(1) AS count " +
            "FROM epiflu " +
            "<where> " +
            "1=1 " +
            "AND is_china = 1 " +
            "<if test=\"chinaInputVO.province != null and chinaInputVO.province != ''\"> " +
            "AND china_city = #{chinaInputVO.province} " +
            "</if> " +
            "<if test=\"chinaInputVO.clade != null and chinaInputVO.hasValidClade()\"> " +
            "AND clade in " +
            "<foreach collection=\"chinaInputVO.clade\" item=\"item\" open=\"(\" close=\")\" separator=\",\"> " +
            "#{item}" +
            "</foreach>" +
            "</if> " +
            "<if test=\"chinaInputVO.subtype != null and chinaInputVO.subtype != ''\"> " +
            "AND subtype = #{chinaInputVO.subtype} " +
            "</if> " +
            "<if test=\"chinaInputVO.startDate != null and chinaInputVO.startDate != ''\"> " +
            "AND collection_date >= #{chinaInputVO.startDate}::date " +
            "</if> " +
            "<if test=\"chinaInputVO.endDate != null and chinaInputVO.endDate != ''\"> " +
            "AND collection_date &lt;= #{chinaInputVO.endDate}::date " +
            "</if> " +
            "</where> " +
            "GROUP BY subtype;" +
            "</script>")
    List<ChinaEpifluDTO> findChinaEpiflu(@Param("chinaInputVO") ChinaInputVO chinaInputVO);


    /**
     * 查询指定洲的clade分布
     * @param continent 洲名称
     * @return clade统计列表
     */
    @Select("SELECT clade, COUNT(*) AS count " +
            "FROM public.epiflu " +
            "WHERE china_city = #{continent} " +
            "AND subtype = #{subtype} " +
            "GROUP BY clade " +
            "ORDER BY count DESC")
    List<CladeCountResult> selectCladeCountByProvince(@Param("continent") String continent , @Param("subtype") String subType);


    @Select("SELECT COUNT(1), MAX(collection_date) AS MAXCollectionDate,MIN(collection_date) AS MINCollectionDate " +
            "FROM epiflu WHERE subtype = #{subtype} AND is_china = 1")
    ChinaCollectionDateDTO findChinaCollectionDate(@Param("subtype") String subtype);

    @Select("<script>" +
            "SELECT " +
            "COUNT(1) AS count " +
            "FROM epiflu " +
            "<where> " +
            "AND is_china = 1 " +
            "<if test=\"chinaInputVO.province != null and chinaInputVO.province != ''\"> " +
            "AND china_city = #{chinaInputVO.province} " +
            "</if> " +
            "<if test=\"chinaInputVO.clade != null and chinaInputVO.hasValidClade()\"> " +
            "AND clade in " +
            "<foreach collection=\"chinaInputVO.clade\" item=\"item\" open=\"(\" close=\")\" separator=\",\"> " +
            "#{item}" +
            "</foreach>" +
            "</if> " +
            "<if test=\"chinaInputVO.subtype != null and chinaInputVO.subtype != ''\"> " +
            "AND subtype = #{chinaInputVO.subtype} " +
            "</if> " +
            "<if test=\"chinaInputVO.startDate != null and chinaInputVO.startDate != ''\"> " +
            "AND collection_date >= #{chinaInputVO.startDate}::date " +
            "</if> " +
            "<if test=\"chinaInputVO.endDate != null and chinaInputVO.endDate != ''\"> " +
            "AND collection_date &lt;= #{chinaInputVO.endDate}::date " +
            "</if> " +
            "</where>;"+
            "</script>")
    Integer selectCountByChinaInput(@Param("chinaInputVO") ChinaInputVO chinaInputVO);


    @Select("SELECT COUNT(1), MAX(collection_date) AS MAXCollectionDate,MIN(collection_date) AS MINCollectionDate " +
            "FROM epiflu WHERE subtype = #{subtype}")
    ChinaCollectionDateDTO findCollectionDate(@Param("subtype") String subtype);

    @Select("<script>" +
            "SELECT " +
            "COUNT(1) AS count " +
            "FROM epiflu " +
            "<where> " +
            "AND 1 = 1 " +
            "<if test=\"iputVO.continent != null and iputVO.continent != ''\"> " +
            "AND continent_location = #{iputVO.continent} " +
            "</if> " +
            "<if test=\"iputVO.clade != null and iputVO.hasValidClade()\"> " +
            "AND clade in " +
            "<foreach collection=\"iputVO.clade\" item=\"item\" open=\"(\" close=\")\" separator=\",\"> " +
            "#{item}" +
            "</foreach>" +
            "</if> " +
            "<if test=\"iputVO.subtype != null and iputVO.subtype != ''\"> " +
            "AND subtype = #{iputVO.subtype} " +
            "</if> " +
            "<if test=\"iputVO.startDate != null and iputVO.startDate != ''\"> " +
            "AND collection_date >= #{iputVO.startDate}::date " +
            "</if> " +
            "<if test=\"iputVO.endDate != null and iputVO.endDate != ''\"> " +
            "AND collection_date &lt;= #{iputVO.endDate}::date " +
            "</if> " +
            "</where>;"+
            "</script>")
    Integer selectCountByInput(@Param("iputVO") InputVO iputVO);

}