package com.qizhen.healsphere.web.param;

public enum ChinaProvinceCoordinates {
    BEIJING("+39.9042", "+116.4074"),
    TIANJIN("+39.0852", "+117.2069"),
    HEBEI("+38.0426", "+114.5086"),
    SHANXI("+37.8707", "+112.5493"),
    NEIMENGGU("+40.8184", "+111.6701"),
    LIAONING("+41.7967", "+123.4290"),
    JILIN("+43.8961", "+125.3245"),
    HEILONGJIANG("+45.7588", "+126.6424"),
    SHANGHAI("+31.2304", "+121.4737"),
    JIANGSU("+32.0415", "+118.7674"),
    ZHEJIANG("+30.2741", "+120.1551"),
    ANHUI("+31.8257", "+117.2264"),
    FUJIAN("+26.0789", "+119.3062"),
    JIANGXI("+28.6764", "+115.8921"),
    SHANDONG("+36.6519", "+117.1901"),
    HENAN("+34.7565", "+113.6654"),
    HUBEI("+30.5928", "+114.3055"),
    HUNAN("+28.2184", "+112.9823"),
    GUANGDONG("+23.1291", "+113.2644"),
    GUANGXI("+22.8240", "+108.2402"),
    HAINAN("+20.0169", "+110.3498"),
    CHONGQING("+29.5630", "+106.5504"),
    SICHUAN("+30.5728", "+104.0668"),
    GUIZHOU("+26.5783", "+106.7134"),
    YUNNAN("+25.0406", "+102.7122"),
    TIBET("+29.6442", "+91.1327"),
    SHANXI_WEST("+34.2632", "+108.9480"),
    GANSU("+36.0581", "+103.8343"),
    QINGHAI("+36.6250", "+101.7789"),
    NINGXIA("+38.4864", "+106.2324"),
    XINJIANG("+43.8258", "+87.6178"),
    HONG_KONG("+22.3193", "+114.1694"),
    MACAO("+22.1987", "+113.5439"),
    TAIWAN("+23.6978", "+120.9605"),
    SHAANXI("34.2655", "108.9487");

    private final String latitude;
    private final String longitude;

    // 构造函数
    ChinaProvinceCoordinates(String latitude, String longitude) {
        this.latitude = latitude;
        this.longitude = longitude;
    }

    // 获取纬度
    public String getLatitude() {
        return latitude;
    }

    // 获取经度
    public String getLongitude() {
        return longitude;
    }

    // 根据省级行政区名称获取枚举实例
    public static ChinaProvinceCoordinates fromName(String name) {
        // 特殊处理 新疆维吾尔族自治区->新疆、广西壮族自治区->广西
        if ("XINJIANG_UYGHUR_AUTONOMOUS_REGION".equalsIgnoreCase(name)) {
            return XINJIANG;
        }
        if ("GUANGXI_ZHUANG_AUTONOMOUS_REGION".equalsIgnoreCase(name)) {
            return GUANGXI;
        }
        for (ChinaProvinceCoordinates province : ChinaProvinceCoordinates.values()) {
            if (province.name().equalsIgnoreCase(name)) {
                return province;
            }
        }
        throw new IllegalArgumentException("Unknown province: " + name);
    }

    // 重写 toString 方法，方便打印
    @Override
    public String toString() {
        return name() + " - Latitude: " + latitude + ", Longitude: " + longitude;
    }
}